/**
 * New node file
 */
var glob = require('glob');
var EventHandler = require('../../util/eventhandler');
var fs = require('fs-extra');
var exec = require('child_process').exec;
var path = require('path');
var log4js = require('log4js');
var logger;

function ChromeCrashWatchdog() {

	this.closeChromeTimeoutHandler;
	this.isrunning = false;
	var currentIndex = 0;
	var doCloseChrome = false;
	logger = log4js.getLogger('watchdog');
	logger.setLevel('WARN');

	this.execute = function(file,backupfile,callback) {
		if (file === undefined){
			file = '/var/log/core_dump/chrome*.core';
		}
		if (backupfile === undefined){
			backupfile = '/apps/nexus2/Logs/chrome-crash.log';
		}

		if (this.isrunning === true){
			return;
		}

		this.isrunning = true;
		//console.log('executing chrome crash watch dog');
		this.isFileNameExists(file, EventHandler.CreateEventHandler(this, this.onDoneCheckNameHandler,{backup:backupfile,callback:callback}));

	};

	this.onDoneCheckNameHandler = function(calle, state, files) {
		calle.onDoneCheckName(files,state.backup,state.callback);
	};

	this.onDoneCheckName = function(files,backupfile,callback) {
		currentIndex = 0;
		doCloseChrome = false;

		this.startCheckGDB(files,callback);
	};

	this.startCheckGDB = function(files,callback){
		var file = files[currentIndex];
		var owner = this;
		if (file){
			logger.warn('Checking file:' + file);
			exec("gdb -q /opt/google/chrome/chrome " + file + " --batch",function(err,stdout,stderr){
				if (stdout) {
					if (stdout.toLowerCase().indexOf('glxcreatenewcontext') > -1) {
						logger.warn('glx error. deleting core dump only');
						owner.deleteFile(file);
					}else{
						logger.warn('vlc error. deleting core dump & restarting chrome');
						owner.deleteFile(file);
						doCloseChrome = true;
					}
				}
				currentIndex++;
				owner.startCheckGDB(files);
			});
		}else{
			this.onDone(callback,'');
		}
	}

	this.onFileCopiedHandler = function(calle, state, success) {
		calle.onFileCopied(state.file,state.callback);
	};

	this.onFileCopied = function(file,callback){
		this.deleteFile(file,EventHandler.CreateEventHandler(this,this.onFileDeletedHandler,{file:file,callback:callback}));
	};

	this.onFileDeletedHandler = function(calle,state,success){
		calle.onFileDeleted(state.file,state.callback,success);
	};

	this.onFileDeleted = function(file,callback,success){
		clearTimeout(this.closeChromeTimeoutHandler);
		this.closeChromeTimeoutHandler = setTimeout(EventHandler.CreateEventHandler(this,this.doCloseChromeHandler,callback),4000);
	};

	this.doCloseChromeHandler = function(calle,callback){

		calle.closeChrome();
		calle.onDone(callback,'chromerestarted');
	};

	this.onDone = function(callback,message){

		if (doCloseChrome == true){
			this.closeChrome();
		}

		this.isrunning = false;
		if (callback){
			callback(message);
		}
	}

	this.copyFile = function(source, targetFile, callback) {
		fs.mkdir(path.dirname(targetFile), function(err) {
			fs.copy(source, targetFile, function(err) {
				var res = true;
				if (err) {
					res = false;
				}
				callback(res);
			});
		});
	};

	this.closeChrome = function() {
		logger.warn('Flagging download manager to restart chrome');
		exec('touch /apps/nexus2/restart-chrome', function(error, stdout, stderr) {
		});
	};

	this.deleteFile = function(filename, callback) {
		logger.warn('deleting ' + filename);
		fs.unlink(filename, function(err) {
			var success = true;
			if (err !== null && err !== undefined) {
				logger.error('error while deleting ' + filename);
				logger.error(err);
				success = false;
			}
			if (callback !== null && callback !== undefined) {
				callback(success);
			}
		});
	};

	this.isFileNameExists = function(name, callback) {
		glob(name, null, function(err, files) {
			if (err !== null && err !== undefined){
				files = [];
				console.log('error while glob');
				console.log(err);
			}
			callback(files);
		});
	};
}

module.exports = ChromeCrashWatchdog;
